//Functions for managing the main application cycle, registering the callbacks
 // to create an application and its completion
#include <app.h>

// Functions for creating and managing user interface elements. Here are all the widgets,
// such as: windows, buttons, etc.
#include <Elementary.h>

//Functions for using callback for hardware Back button and for
// using extended widgets for a circle screen
#include <efl_extension.h>

typedef struct _UIData {
   Evas_Object *win;
   Evas_Object *conform;
   Evas_Object *scroller;
   Evas_Object *image;
} UIData;

static void
_win_back_cb(void *data, Evas_Object *obj, void *event_info)
{
   UIData *ui = data;

   elm_win_lower(ui->win);
}

static void
_file_abs_resource_path_get(char *res_file_path, char *abs_path, int buf_size)
{
   char *res_dir_path = app_get_resource_path();

   if (res_dir_path)
     {
        snprintf(abs_path, buf_size, "%s%s", res_dir_path, res_file_path);
        free(res_dir_path);
     }
}

static void
_image_create(UIData *ui)
{
   ui->image = elm_image_add(ui->scroller);

   char abs_path_to_image[PATH_MAX] = {0,};
   _file_abs_resource_path_get("images/image_big.png", abs_path_to_image, PATH_MAX);
   elm_image_file_set(ui->image, abs_path_to_image, NULL);

   Evas_Object *evas_image = elm_image_object_get(ui->image);
   int w = 0;
   int h = 0;
   evas_object_image_size_get(evas_image, &w, &h);

   evas_object_size_hint_min_set(ui->image, w, h);
   evas_object_size_hint_max_set(ui->image, w, h);

   Eina_Bool is_animated = elm_image_animated_available_get(ui->image);
   if (is_animated)
     {
        elm_image_animated_set(ui->image, EINA_TRUE);
        elm_image_animated_play_set(ui->image, EINA_TRUE);
     }

   evas_object_show(ui->image);
}

static void
_scroller_create(UIData *ui)
{
   ui->scroller = elm_scroller_add(ui->conform);

   _image_create(ui);
   elm_object_content_set(ui->scroller, ui->image);

   evas_object_show(ui->scroller);
}

static void
_conformant_create(UIData *ui)
{
   ui->conform = elm_conformant_add(ui->win);

   //Specify to conformant occupies all allocated to him space
      // (horizontally and vertically)
   evas_object_size_hint_weight_set(ui->conform, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

   _scroller_create(ui);
   elm_object_content_set(ui->conform, ui->scroller);

   evas_object_show(ui->conform);
}

static void
_window_create(UIData *ui)
{
	// Create and configure the main window
   ui->win = elm_win_util_standard_add(NULL, NULL);

   // Create a conformant - in this case, the main container in the application,
  // which is also the interface between the application and
  // system elements, such as a keyboard.
   _conformant_create(ui);

   //  Set the size of the conformant widget the same as the size of the window
   elm_win_resize_object_add(ui->win, ui->conform);

   //Register the function that will be called
    //when you press the hardware Back button
   eext_object_event_callback_add(ui->win, EEXT_CALLBACK_BACK, _win_back_cb, ui);

   // Always display the window only after the entire base
       // user interface will be displayed.
   evas_object_show(ui->win);
}

static bool
_app_create_cb(void *data)
{
   UIData *ui = data;

   _window_create(ui);

   return true;
}

static void
_app_terminate_cb(void *data)
{
   UIData *ui = data;


   //Before closing the application, delete the main widget (window),
    // then all  "children" widgets of this window (all widgets of this application) will be deleted.
   evas_object_del(ui->win);
}

int
main(int argc, char *argv[])
{
   UIData ui = {0,};
   ui_app_lifecycle_callback_s lifecycle_callbacks = {0,};

   //Register created functions to transfer them to the main loop of
    // this application.
   lifecycle_callbacks.create    = _app_create_cb;
   lifecycle_callbacks.terminate = _app_terminate_cb;
   // Calling the next function, launch the main application cycle
   return ui_app_main(argc, argv, &lifecycle_callbacks, &ui);
}
